def _quote_escape(s):
    return s.replace('\\', '\\\\').replace('"', '\\"')

def _serialize(o, prefix, indent, eoln):

    pf = prefix+indent
    s = ""

    if o is None:
        s = "null"

    elif isinstance(o, bool):
        s = "true" if o else "false"

    elif isinstance(o, (int, float)):
        s = str(o)

    elif isinstance(o, str):
        s = f'"{_quote_escape(o)}"'

    elif isinstance(o, list):
        s = "[" + eoln
        comma = ""
        for v in o:
            s = s + comma + pf
            s += _serialize(v, pf+indent, indent, eoln)
            comma = "," + eoln
        s = s + eoln + prefix + "]"

    elif isinstance(o, set):
        s = "{" + eoln
        comma = ""
        for v in o:
            s = s + comma + pf
            s += _serialize(v, pf+indent, indent, eoln)
            comma = "," + eoln
        s = s + eoln + prefix + "}"

    elif isinstance(o, dict):
        s = "{" + eoln
        comma = ""
        for k,v in o.items():
            s = s + comma + pf
            esc = _quote_escape(k)
            s += (f'"{esc}"' if ' ' in esc else esc) + ": "
            s += _serialize(v, pf+indent, indent, eoln)
            comma = "," + eoln
        s = s + eoln + prefix + "}"

    return s

def dumps(o, indent="  ", eoln="\n"):
    return _serialize(o, "", indent, eoln)



from collections import namedtuple

from .lark_parser import Lark_StandAlone, Transformer

class TJawnTransformer(Transformer):
    Pair = namedtuple('Pair', ['k', 'v'])

    null = lambda self, _: None
    true = lambda self, _: True
    false = lambda self, _: False

    def number(self, n):
        (n,) = n
        return float(n)

    def string(self, s):
        (s,) = s
        s = str(s[1:-1]).replace('\\"', '"')
        return s

    def cname(self, s):
        (s,) = s
        return str(s)

    def text(self, s):
        (s,) = s
        s = str(s[3:-3]).replace('\\"', '"')
        return s

    def list(self, s):
        if len(s) == 1 and s[0] is None:
            return list()
        return list(s)

    def set(self, s):
        if len(s) == 1 and s[0] is None:
            return set()
        return set(s)

    def dict(self, s):
        if len(s) == 1 and s[0] is None:
            return dict()
        return {p.k: p.v for p in s}

    def file(self, s):
        return self.dict(s)

    def key(self, s):
        (s, ) = s
        k = str(s) if s.type == 'CNAME' else str(s[1:-1]).replace('\\"', '"')
        return k

    def pair(self, s):
        return self.Pair(s[0], s[1])

parser = Lark_StandAlone(transformer=TJawnTransformer())

def loads(text: str):
    return parser.parse(text)
